#
# Ronin Exploits - A Ruby library for Ronin that provides exploitation and
# payload crafting functionality.
#
# Copyright (c) 2007-2009 Hal Brodigan (postmodern.mod3 at gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

require 'ronin/arch'

require 'extlib'

module Ronin
  module Model
    module TargetsArch
      def self.included(base)
        base.module_eval do
          # The targeted architecture
          belongs_to :arch,
                     :nullable => true

          #
          # The targeted architecture.
          #
          # @param [Symbol] name
          #   If a name is given, a new Arch with the matching name will be
          #   targeted.
          #
          # @return [Arch]
          #   The architecture to target.
          #
          # @example Accessing the targeted architecture
          #   target.arch
          #   # => nil
          #
          # @example Setting the targeted architecture
          #   target.arch :i686
          #   # => #<Ronin::Arch type=Ronin::Arch id=nil name="i686"
          #   # endian="little" address_length=4>
          #
          def arch(name=nil)
            if name
              return self.arch = Arch[name]
            else
              return relationships[:arch].get(self)
            end
          end
        end

        model_name = base.name.split('::').last.snake_case.plural.to_sym
        Arch.has Arch.n, model_name, :model => base.name
      end
    end
  end
end
